@@ -44,6 +44,7 @@
 // Copyright (C) 2020 Kai Pastor <dg0yt@darc.de>
 // Copyright (C) 2021, 2022 Stefan Löffler <st.loeffler@gmail.com>
 // Copyright (C) 2021 sunderme <sunderme@gmx.de>
+// Copyright (C) 2022 Even Rouault <even.rouault@spatialys.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -77,9 +78,7 @@
 #    include <fontconfig/fontconfig.h>
 #endif
 
-#ifdef _MSC_VER
-#    define strcasecmp stricmp
-#else
+#ifndef _MSC_VER
 #    include <strings.h>
 #endif
 
@@ -91,6 +90,9 @@
 #include "UnicodeMapTables.h"
 #include "UnicodeMapFuncs.h"
 
+#include "fofi/FoFiTrueType.h"
+#include "fofi/FoFiIdentifier.h"
+
 //------------------------------------------------------------------------
 
 #define cidToUnicodeCacheSize 4
@@ -250,10 +252,12 @@ public:
     ~SysFontList();
     SysFontList(const SysFontList &) = delete;
     SysFontList &operator=(const SysFontList &) = delete;
-    const SysFontInfo *find(const GooString *name, bool isFixedWidth, bool exact);
+    const SysFontInfo *find(const std::string &name, bool isFixedWidth, bool exact);
+
+    const std::vector<SysFontInfo *> &getFonts() const { return fonts; }
 
 #ifdef _WIN32
-    void scanWindowsFonts(GooString *winFontDir);
+    void scanWindowsFonts(const std::string &winFontDir);
 #endif
 #ifdef WITH_FONTCONFIGURATION_FONTCONFIG
     void addFcFont(SysFontInfo *si) { fonts.push_back(si); }
@@ -275,13 +279,13 @@ SysFontList::~SysFontList()
     }
 }
 
-const SysFontInfo *SysFontList::find(const GooString *name, bool fixedWidth, bool exact)
+const SysFontInfo *SysFontList::find(const std::string &name, bool fixedWidth, bool exact)
 {
     GooString *name2;
     bool bold, italic, oblique;
     int n;
 
-    name2 = name->copy();
+    name2 = new GooString(name);
 
     // remove space, comma, dash chars
     {
@@ -582,8 +586,9 @@ Unicode GlobalParams::mapNameToUnicodeAll(const char *charName)
 {
     // no need to lock - nameToUnicodeZapfDingbats and nameToUnicodeText are constant
     Unicode u = nameToUnicodeZapfDingbats->lookup(charName);
-    if (!u)
+    if (!u) {
         u = nameToUnicodeText->lookup(charName);
+    }
     return u;
 }
 
@@ -660,13 +665,15 @@ static bool findModifier(const char *name, const char *modifier, const char **st
 {
     const char *match;
 
-    if (name == nullptr)
+    if (name == nullptr) {
         return false;
+    }
 
     match = strstr(name, modifier);
     if (match) {
-        if (*start == nullptr || match < *start)
+        if (*start == nullptr || match < *start) {
             *start = match;
+        }
         return true;
     } else {
         return false;
@@ -681,28 +688,30 @@ static const char *getFontLang(const GfxFont *font)
     if (font->isCIDFont()) {
         const GooString *collection = ((GfxCIDFont *)font)->getCollection();
         if (collection) {
-            if (strcmp(collection->c_str(), "Adobe-GB1") == 0)
+            if (strcmp(collection->c_str(), "Adobe-GB1") == 0) {
                 lang = "zh-cn"; // Simplified Chinese
-            else if (strcmp(collection->c_str(), "Adobe-CNS1") == 0)
+            } else if (strcmp(collection->c_str(), "Adobe-CNS1") == 0) {
                 lang = "zh-tw"; // Traditional Chinese
-            else if (strcmp(collection->c_str(), "Adobe-Japan1") == 0)
+            } else if (strcmp(collection->c_str(), "Adobe-Japan1") == 0) {
                 lang = "ja"; // Japanese
-            else if (strcmp(collection->c_str(), "Adobe-Japan2") == 0)
+            } else if (strcmp(collection->c_str(), "Adobe-Japan2") == 0) {
                 lang = "ja"; // Japanese
-            else if (strcmp(collection->c_str(), "Adobe-Korea1") == 0)
+            } else if (strcmp(collection->c_str(), "Adobe-Korea1") == 0) {
                 lang = "ko"; // Korean
-            else if (strcmp(collection->c_str(), "Adobe-UCS") == 0)
+            } else if (strcmp(collection->c_str(), "Adobe-UCS") == 0) {
                 lang = "xx";
-            else if (strcmp(collection->c_str(), "Adobe-Identity") == 0)
+            } else if (strcmp(collection->c_str(), "Adobe-Identity") == 0) {
                 lang = "xx";
-            else {
+            } else {
                 error(errUnimplemented, -1, "Unknown CID font collection, please report to poppler bugzilla.");
                 lang = "xx";
             }
-        } else
+        } else {
             lang = "xx";
-    } else
+        }
+    } else {
         lang = "xx";
+    }
     return lang;
 }
 
@@ -717,31 +726,39 @@ static FcPattern *buildFcPattern(const GfxFont *font, const GooString *base14Nam
     char *fontName = strdup(((base14Name == nullptr) ? font->getNameWithoutSubsetTag() : base14Name->toStr()).c_str());
 
     const char *modifiers = strchr(fontName, ',');
-    if (modifiers == nullptr)
+    if (modifiers == nullptr) {
         modifiers = strchr(fontName, '-');
+    }
 
     // remove the - from the names, for some reason, Fontconfig does not
     // understand "MS-Mincho" but does with "MS Mincho"
     const int len = strlen(fontName);
-    for (int i = 0; i < len; i++)
+    for (int i = 0; i < len; i++) {
         fontName[i] = (fontName[i] == '-' ? ' ' : fontName[i]);
+    }
 
     start = nullptr;
     findModifier(modifiers, "Regular", &start);
     findModifier(modifiers, "Roman", &start);
 
-    if (findModifier(modifiers, "Oblique", &start))
+    if (findModifier(modifiers, "Oblique", &start)) {
         slant = FC_SLANT_OBLIQUE;
-    if (findModifier(modifiers, "Italic", &start))
+    }
+    if (findModifier(modifiers, "Italic", &start)) {
         slant = FC_SLANT_ITALIC;
-    if (findModifier(modifiers, "Bold", &start))
+    }
+    if (findModifier(modifiers, "Bold", &start)) {
         weight = FC_WEIGHT_BOLD;
-    if (findModifier(modifiers, "Light", &start))
+    }
+    if (findModifier(modifiers, "Light", &start)) {
         weight = FC_WEIGHT_LIGHT;
-    if (findModifier(modifiers, "Medium", &start))
+    }
+    if (findModifier(modifiers, "Medium", &start)) {
         weight = FC_WEIGHT_MEDIUM;
-    if (findModifier(modifiers, "Condensed", &start))
+    }
+    if (findModifier(modifiers, "Condensed", &start)) {
         width = FC_WIDTH_CONDENSED;
+    }
 
     if (start) {
         // There have been "modifiers" in the name, crop them to obtain
@@ -755,12 +772,15 @@ static FcPattern *buildFcPattern(const GfxFont *font, const GooString *base14Nam
     }
 
     // use font flags
-    if (font->isFixedWidth())
+    if (font->isFixedWidth()) {
         spacing = FC_MONO;
-    if (font->isBold())
+    }
+    if (font->isBold()) {
         weight = FC_WEIGHT_BOLD;
-    if (font->isItalic())
+    }
+    if (font->isItalic()) {
         slant = FC_SLANT_ITALIC;
+    }
 
     bool freeFamily = true;
     // if the FontDescriptor specified a family name use it
@@ -839,28 +859,33 @@ static FcPattern *buildFcPattern(const GfxFont *font, const GooString *base14Nam
     const char *lang = getFontLang(font);
 
     p = FcPatternBuild(nullptr, FC_FAMILY, FcTypeString, family, FC_LANG, FcTypeString, lang, NULL);
-    if (slant != -1)
+    if (slant != -1) {
         FcPatternAddInteger(p, FC_SLANT, slant);
-    if (weight != -1)
+    }
+    if (weight != -1) {
         FcPatternAddInteger(p, FC_WEIGHT, weight);
-    if (width != -1)
+    }
+    if (width != -1) {
         FcPatternAddInteger(p, FC_WIDTH, width);
-    if (spacing != -1)
+    }
+    if (spacing != -1) {
         FcPatternAddInteger(p, FC_SPACING, spacing);
+    }
 
-    if (freeFamily)
+    if (freeFamily) {
         free((char *)family);
+    }
     return p;
 }
 #endif
 
-GooString *GlobalParams::findFontFile(const GooString *fontName)
+GooString *GlobalParams::findFontFile(const std::string &fontName)
 {
     GooString *path = nullptr;
 
     setupBaseFonts(POPPLER_FONTSDIR);
     globalParamsLocker();
-    const auto fontFile = fontFiles.find(fontName->toStr());
+    const auto fontFile = fontFiles.find(fontName);
     if (fontFile != fontFiles.end()) {
         path = new GooString(fontFile->second);
     }
@@ -868,6 +893,43 @@ GooString *GlobalParams::findFontFile(const GooString *fontName)
     return path;
 }
 
+#if defined(WITH_FONTCONFIGURATION_FONTCONFIG) || defined(WITH_FONTCONFIGURATION_WIN32)
+
+static bool supportedFontForEmbedding(Unicode uChar, const char *filepath, int faceIndex)
+{
+    if (!GooString::endsWith(filepath, ".ttf") && !GooString::endsWith(filepath, ".ttc") && !GooString::endsWith(filepath, ".otf")) {
+        // for now we only support ttf, ttc, otf fonts
+        return false;
+    }
+
+    const FoFiIdentifierType fontFoFiType = FoFiIdentifier::identifyFile(filepath);
+    if (fontFoFiType != fofiIdTrueType && fontFoFiType != fofiIdTrueTypeCollection && fontFoFiType != fofiIdOpenTypeCFF8Bit && fontFoFiType != fofiIdOpenTypeCFFCID) {
+        // for now we only support ttf, ttc, otf fonts
+        return false;
+    }
+
+    const std::unique_ptr<FoFiTrueType> fft = FoFiTrueType::load(filepath, faceIndex);
+    if (!fft) {
+        error(errIO, -1, "Form::addFontToDefaultResources. Failed to FoFiTrueType::load %s", filepath);
+        return false;
+    }
+
+    // Look for the Unicode BMP cmaps, which are 0/3 or 3/1
+    int unicodeBMPCMap = fft->findCmap(0, 3);
+    if (unicodeBMPCMap < 0) {
+        unicodeBMPCMap = fft->findCmap(3, 1);
+    }
+    if (unicodeBMPCMap < 0) {
+        // for now we only support files with unicode bmp cmaps
+        return false;
+    }
+
+    const int glyph = fft->mapCodeToGID(unicodeBMPCMap, uChar);
+    return glyph > 0;
+}
+
+#endif
+
 /* if you can't or don't want to use Fontconfig, you need to implement
    this function for your platform. For Windows, it's in GlobalParamsWin.cc
 */
@@ -888,14 +950,15 @@ GooString *GlobalParams::findSystemFontFile(const GfxFont *font, SysFontType *ty
     const SysFontInfo *fi = nullptr;
     FcPattern *p = nullptr;
     GooString *path = nullptr;
-    const GooString *fontName = font->getName();
+    const std::optional<std::string> &fontName = font->getName();
     GooString substituteName;
-    if (!fontName)
+    if (!fontName) {
         return nullptr;
+    }
 
     globalParamsLocker();
 
-    if ((fi = sysFonts->find(fontName, font->isFixedWidth(), true))) {
+    if ((fi = sysFonts->find(*fontName, font->isFixedWidth(), true))) {
         path = fi->path->copy();
         *type = fi->type;
         *fontNum = fi->fontNum;
@@ -909,13 +972,15 @@ GooString *GlobalParams::findSystemFontFile(const GfxFont *font, SysFontType *ty
         FcLangSet *lb = nullptr;
         p = buildFcPattern(font, base14Name);
 
-        if (!p)
+        if (!p) {
             goto fin;
+        }
         FcConfigSubstitute(nullptr, p, FcMatchPattern);
         FcDefaultSubstitute(p);
         set = FcFontSort(nullptr, p, FcFalse, nullptr, &res);
-        if (!set)
+        if (!set) {
             goto fin;
+        }
 
         // find the language we want the font to support
         const char *lang = getFontLang(font);
@@ -932,8 +997,9 @@ GooString *GlobalParams::findSystemFontFile(const GfxFont *font, SysFontType *ty
         while (fi == nullptr) {
             for (i = 0; i < set->nfont; ++i) {
                 res = FcPatternGetString(set->fonts[i], FC_FILE, 0, &s);
-                if (res != FcResultMatch || !s)
+                if (res != FcResultMatch || !s) {
                     continue;
+                }
                 if (lb != nullptr) {
                     FcLangSet *l;
                     res = FcPatternGetLangSet(set->fonts[i], FC_LANG, 0, &l);
@@ -963,8 +1029,9 @@ GooString *GlobalParams::findSystemFontFile(const GfxFont *font, SysFontType *ty
                     }
                 }
                 ext = strrchr((char *)s, '.');
-                if (!ext)
+                if (!ext) {
                     continue;
+                }
                 if (!strncasecmp(ext, ".ttf", 4) || !strncasecmp(ext, ".ttc", 4) || !strncasecmp(ext, ".otf", 4)) {
                     int weight, slant;
                     bool bold = font->isBold();
@@ -975,14 +1042,16 @@ GooString *GlobalParams::findSystemFontFile(const GfxFont *font, SysFontType *ty
                     if (weight == FC_WEIGHT_DEMIBOLD || weight == FC_WEIGHT_BOLD || weight == FC_WEIGHT_EXTRABOLD || weight == FC_WEIGHT_BLACK) {
                         bold = true;
                     }
-                    if (slant == FC_SLANT_ITALIC)
+                    if (slant == FC_SLANT_ITALIC) {
                         italic = true;
-                    if (slant == FC_SLANT_OBLIQUE)
+                    }
+                    if (slant == FC_SLANT_OBLIQUE) {
                         oblique = true;
+                    }
                     *fontNum = 0;
                     *type = (!strncasecmp(ext, ".ttc", 4)) ? sysFontTTC : sysFontTTF;
                     FcPatternGetInteger(set->fonts[i], FC_INDEX, 0, fontNum);
-                    SysFontInfo *sfi = new SysFontInfo(fontName->copy(), bold, italic, oblique, font->isFixedWidth(), new GooString((char *)s), *type, *fontNum, substituteName.copy());
+                    SysFontInfo *sfi = new SysFontInfo(new GooString(*fontName), bold, italic, oblique, font->isFixedWidth(), new GooString((char *)s), *type, *fontNum, substituteName.copy());
                     sysFonts->addFcFont(sfi);
                     fi = sfi;
                     path = new GooString((char *)s);
@@ -996,19 +1065,22 @@ GooString *GlobalParams::findSystemFontFile(const GfxFont *font, SysFontType *ty
                     if (weight == FC_WEIGHT_DEMIBOLD || weight == FC_WEIGHT_BOLD || weight == FC_WEIGHT_EXTRABOLD || weight == FC_WEIGHT_BLACK) {
                         bold = true;
                     }
-                    if (slant == FC_SLANT_ITALIC)
+                    if (slant == FC_SLANT_ITALIC) {
                         italic = true;
-                    if (slant == FC_SLANT_OBLIQUE)
+                    }
+                    if (slant == FC_SLANT_OBLIQUE) {
                         oblique = true;
+                    }
                     *fontNum = 0;
                     *type = (!strncasecmp(ext, ".pfa", 4)) ? sysFontPFA : sysFontPFB;
                     FcPatternGetInteger(set->fonts[i], FC_INDEX, 0, fontNum);
-                    SysFontInfo *sfi = new SysFontInfo(fontName->copy(), bold, italic, oblique, font->isFixedWidth(), new GooString((char *)s), *type, *fontNum, substituteName.copy());
+                    SysFontInfo *sfi = new SysFontInfo(new GooString(*fontName), bold, italic, oblique, font->isFixedWidth(), new GooString((char *)s), *type, *fontNum, substituteName.copy());
                     sysFonts->addFcFont(sfi);
                     fi = sfi;
                     path = new GooString((char *)s);
-                } else
+                } else {
                     continue;
+                }
                 break;
             }
             if (lb != nullptr) {
@@ -1021,7 +1093,7 @@ GooString *GlobalParams::findSystemFontFile(const GfxFont *font, SysFontType *ty
         }
         FcFontSetDestroy(set);
     }
-    if (path == nullptr && (fi = sysFonts->find(fontName, font->isFixedWidth(), false))) {
+    if (path == nullptr && (fi = sysFonts->find(*fontName, font->isFixedWidth(), false))) {
         path = fi->path->copy();
         *type = fi->type;
         *fontNum = fi->fontNum;
@@ -1030,23 +1102,103 @@ GooString *GlobalParams::findSystemFontFile(const GfxFont *font, SysFontType *ty
         substituteFontName->Set(substituteName.c_str());
     }
 fin:
-    if (p)
+    if (p) {
         FcPatternDestroy(p);
+    }
 
     return path;
 }
 
+FamilyStyleFontSearchResult GlobalParams::findSystemFontFileForFamilyAndStyle(const std::string &fontFamily, const std::string &fontStyle)
+{
+    FcChar8 *fcFilePath = nullptr;
+    int faceIndex = 0;
+    FcPattern *p = FcPatternBuild(nullptr, FC_FAMILY, FcTypeString, fontFamily.c_str(), FC_STYLE, FcTypeString, fontStyle.c_str(), nullptr);
+    FcConfigSubstitute(nullptr, p, FcMatchPattern);
+    FcDefaultSubstitute(p);
+    if (p) {
+        FcResult res;
+        FcFontSet *set = FcFontSort(nullptr, p, FcFalse, nullptr, &res);
+        if (set) {
+            if (res == FcResultMatch && set->nfont > 0) {
+                FcPatternGetString(set->fonts[0], FC_FILE, 0, &fcFilePath);
+                FcPatternGetInteger(set->fonts[0], FC_INDEX, 0, &faceIndex);
+            }
+            FcFontSetDestroy(set);
+        }
+        FcPatternDestroy(p);
+    }
+
+    if (!fcFilePath) {
+        error(errIO, -1, "Couldn't find font file for %s %s", fontFamily.c_str(), fontStyle.c_str());
+        return {};
+    }
+
+    return FamilyStyleFontSearchResult(reinterpret_cast<char *>(fcFilePath), faceIndex);
+}
+
+UCharFontSearchResult GlobalParams::findSystemFontFileForUChar(Unicode uChar, const GfxFont &fontToEmulate)
+{
+    FcPattern *pattern = buildFcPattern(&fontToEmulate, nullptr);
+
+    FcConfigSubstitute(nullptr, pattern, FcMatchPattern);
+    FcDefaultSubstitute(pattern);
+
+    FcResult result = FcResultMatch;
+    FcFontSet *fontSet = FcFontSort(nullptr, pattern, FcFalse, nullptr, &result);
+    FcPatternDestroy(pattern);
+
+    if (fontSet) {
+        const std::unique_ptr<FcFontSet, void (*)(FcFontSet *)> fontSetDeleter(fontSet, [](FcFontSet *fSet) { FcFontSetDestroy(fSet); });
+        for (int i = 0; i < fontSet->nfont; i++) {
+            FcChar8 *fcFilePath = nullptr;
+            int faceIndex = 0;
+            FcChar8 *fcFamily = nullptr;
+            FcChar8 *fcStyle = nullptr;
+            FcPatternGetString(fontSet->fonts[i], FC_FILE, 0, &fcFilePath);
+            FcPatternGetInteger(fontSet->fonts[i], FC_INDEX, 0, &faceIndex);
+            FcPatternGetString(fontSet->fonts[i], FC_FAMILY, 0, &fcFamily);
+            FcPatternGetString(fontSet->fonts[i], FC_STYLE, 0, &fcStyle);
+            if (!fcFilePath || !fcFamily || !fcStyle) {
+                continue;
+            }
+
+            const char *filepath = reinterpret_cast<char *>(fcFilePath);
+
+            if (supportedFontForEmbedding(uChar, filepath, faceIndex)) {
+                return UCharFontSearchResult(filepath, faceIndex, reinterpret_cast<char *>(fcFamily), reinterpret_cast<char *>(fcStyle));
+            }
+        }
+    }
+
+    return {};
+}
+
 #elif defined(WITH_FONTCONFIGURATION_WIN32)
 #    include "GlobalParamsWin.cc"
 
 GooString *GlobalParams::findBase14FontFile(const GooString *base14Name, const GfxFont *font)
 {
-    return findFontFile(base14Name);
+    return findFontFile(base14Name->toStr());
 }
+
 #else
+
+FamilyStyleFontSearchResult GlobalParams::findSystemFontFileForFamilyAndStyle(const std::string &fontFamily, const std::string &fontStyle)
+{
+    error(errUnimplemented, -1, "GlobalParams::findSystemFontFileForFamilyAndStyle not implemented for this platform");
+    return {};
+}
+
+UCharFontSearchResult GlobalParams::findSystemFontFileForUChar(Unicode uChar, const GfxFont &fontToEmulate)
+{
+    error(errUnimplemented, -1, "GlobalParams::findSystemFontFileForUChar not implemented for this platform");
+    return {};
+}
+
 GooString *GlobalParams::findBase14FontFile(const GooString *base14Name, const GfxFont *font)
 {
-    return findFontFile(base14Name);
+    return findFontFile(base14Name->toStr());
 }
 
 static struct
@@ -1074,8 +1226,6 @@ static const char *displayFontDirs[] = { "/usr/share/ghostscript/fonts", "/usr/l
 
 void GlobalParams::setupBaseFonts(const char *dir)
 {
-    GooString *fontName;
-    GooString *fileName;
     FILE *f;
     int i, j;
 
@@ -1083,32 +1233,29 @@ void GlobalParams::setupBaseFonts(const char *dir)
         if (fontFiles.count(displayFontTab[i].name) > 0) {
             continue;
         }
-        fontName = new GooString(displayFontTab[i].name);
-        fileName = nullptr;
+        std::unique_ptr<GooString> fontName = std::make_unique<GooString>(displayFontTab[i].name);
+        std::unique_ptr<GooString> fileName;
         if (dir) {
-            fileName = appendToPath(new GooString(dir), displayFontTab[i].t1FileName);
+            fileName.reset(appendToPath(new GooString(dir), displayFontTab[i].t1FileName));
             if ((f = openFile(fileName->c_str(), "rb"))) {
                 fclose(f);
             } else {
-                delete fileName;
-                fileName = nullptr;
+                fileName.reset();
             }
         }
         for (j = 0; !fileName && displayFontDirs[j]; ++j) {
-            fileName = appendToPath(new GooString(displayFontDirs[j]), displayFontTab[i].t1FileName);
+            fileName.reset(appendToPath(new GooString(displayFontDirs[j]), displayFontTab[i].t1FileName));
             if ((f = openFile(fileName->c_str(), "rb"))) {
                 fclose(f);
             } else {
-                delete fileName;
-                fileName = nullptr;
+                fileName.reset();
             }
         }
         if (!fileName) {
             error(errConfig, -1, "No display font for '{0:s}'", displayFontTab[i].name);
-            delete fontName;
             continue;
         }
-        addFontFile(fontName, fileName);
+        addFontFile(fontName.get(), fileName.get());
     }
 }
 
@@ -1117,13 +1264,13 @@ GooString *GlobalParams::findSystemFontFile(const GfxFont *font, SysFontType *ty
     const SysFontInfo *fi;
     GooString *path;
 
-    const GooString *fontName = font->getName();
+    const std::optional<std::string> &fontName = font->getName();
     if (!fontName)
         return nullptr;
 
     path = nullptr;
     globalParamsLocker();
-    if ((fi = sysFonts->find(fontName, font->isFixedWidth(), false))) {
+    if ((fi = sysFonts->find(*fontName, font->isFixedWidth(), false))) {
         path = fi->path->copy();
         *type = fi->type;
         *fontNum = fi->fontNum;
